#!/usr/bin/python

import sys
import csv
import re


def output_checkfile(serviceinfo):
    # get the items out of the list
    sysctl_var, sysctl_val = serviceinfo
    # convert variable name to a format suitable for 'id' tags
    sysctl_var_id = re.sub('[-\.]', '_', sysctl_var)

    # if the sysctl value is not present, use the variable template
    if not sysctl_val.strip():
        # open the template and perform the conversions 
        with open("template_sysctl_var", 'r') as templatefile:
            filestring = templatefile.read()
            filestring = filestring.replace("SYSCTLID", sysctl_var_id)
            filestring = filestring.replace("SYSCTLVAR", sysctl_var)
	    # write the check
            with open("./output/sysctl_"  + sysctl_var_id +
                      ".sh", 'w+') as outputfile:
                outputfile.write(filestring)
                outputfile.close()

    else:
        # open the template and perform the conversions
        with open("template_sysctl", 'r') as templatefile:
            filestring = templatefile.read()
            filestring = filestring.replace("SYSCTLID", sysctl_var_id)
            filestring = filestring.replace("SYSCTLVAR", sysctl_var)
            filestring = filestring.replace("SYSCTLVAL", sysctl_val)
            # write the check
            with open("./output/sysctl_" + sysctl_var_id +
                      ".sh", 'w+') as outputfile:
                outputfile.write(filestring)
                outputfile.close()


def main():
    if len(sys.argv) < 2:
        print ("Provide a CSV file containing lines of the format: " +
               "sysctlvariable,sysctlvalue")
        sys.exit(1)
    with open(sys.argv[1], 'r') as csv_file:
        # put the CSV line's items into a list
        sysctl_lines = csv.reader(csv_file)
        for line in sysctl_lines:

            # Skip lines of input file starting with comment '#' character
            if line[0].startswith('#'):
                continue

            output_checkfile(line)

    sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/python2

import sys
import os
import lxml.etree as ET

# This script requires one argument: path to OVAL file to be reviewed
#
# The purpose of this script is to review existing (and newly added)
# <ind:instance> elements of corresponding <ind:textfilecontent54_object>
# OVAL objects for their correctness and enforce uniformity on the way
# <ind:instance> elements are used for particular service / filepath location.
#
# Different UNIX services treat their input files they use differently.
# Some accept just the first value of a concrete option and ignore the
# rest (for example 'sshd' or 'iptables' service). While others (for
# example 'auditd' service wrt to processing /etc/audit/auditd.conf file)
# process the input file sequentially and in case there's different value
# provided for previously used option, the last setting is which will get
# actually used. Yet there also exists executables processing input text
# files containing multiple similar / alternative rows (where at some point
# any of these entries could be used) - for example list of available kernel
# entries to boot in /etc/grub.conf.
#
# The point of this testing script is to enforce that each (existing but
# also in the future added) textfilecontent54_object for that particular
# service would follow the expectations / behavioral pattern typical for
# that service (IOW that each textfilecontent54_object for the 'sshd'
# service will check just the first instance, while each textfilecontent54_object
# for e.g. logrotate service will check all instances).


oval_ns = "http://oval.mitre.org/XMLSchema/oval-definitions-5"

def verify_record_sanity(oval_id, filepath, operation, instance):

    # This list contains manually verified textfilecontent54_objects that are safe
    # based on the way they are defined. Use this list to define exceptions from
    # the following service behavioral patterns.
    MANUALLY_VERIFIED_SAFE_IDS = ['oval:ssg:obj:1312', # check_existence='none_exist'
                                  'oval:ssg:obj:2142', # check_existence='none_exist'
                                  'oval:ssg:obj:2165'  # only one 'password' kw in /etc/grub.conf
                                 ]

    # First check if particular OVAL id is in the list of the manually verified ones
    # If so return PASS
    if oval_id in MANUALLY_VERIFIED_SAFE_IDS:
        return 'PASS'

    # This OVAL id isn't in the list of manually blessed ones => check record sanity
    # based on filepath + instance combination

    # /etc/audit/audit.rules case - first entry wins the system:
    # https://www.redhat.com/archives/linux-audit/2015-January/msg00070.html
    if filepath == '/etc/audit/audit.rules' and operation is None and instance == '1':
        return 'PASS'

    # /etc/audit/auditd.conf case - last option setting wins. To verify use '-f' in
    # EXTRAOPTIONS of /etc/sysconfig/auditd, use two different values for the same auditd.conf
    # option & restart auditd service to see at stderr which setting would be applied
    if filepath == '/etc/audit/auditd.conf' and operation == 'greater than or equal' and instance == '1':
        return 'PASS'

    # /etc/sysctl.conf case - last option setting wins. To verify use two different values
    # for e.g. 'kernel.msgmax' option, reboot the system and verify the new value via:
    # 		# cat /proc/sys/kernel/msgmax
    # command
    if filepath == '/etc/sysctl.conf' and operation == 'greater than or equal' and instance == '1':
        return 'PASS'

    # /etc/ssh/sshd_conf case - first option setting wins. To verify supply 'OPTIONS="-T"' to
    # /etc/sysconfig/sshd, provide two different values for some sshd_config directive, restart sshd
    # and verify the sshd config dump output
    if filepath == '/etc/ssh/sshd_config' and operation is None and instance == '1':
        return 'PASS'

    # /etc/init.d/functions case - last option wins (it's a shell script)
    if filepath == '/etc/init.d/functions' and operation == 'greater than or equal' and instance == '1':
        return 'PASS'

    # /etc/inittab case - first option setting wins. To verify use multiple 'id:[0-5]:initdefault:'
    # with different runlevel, reboot the system & obtain current runlevel value via 'runlevel' command
    # after reboot
    if filepath == '/etc/inittab' and operation is None and instance == '1':
        return 'PASS'

    # /etc/modprobe.conf and /etc/modprobe.d cases - to check first instance is sufficient since OVAL is
    # checking if particular kernel module is set disabled
    if filepath == '/etc/modprobe.conf' or filepath.startswith('/etc/modprobe.d'):
        if operation is None and instance == '1':
            return 'PASS'

    # /etc/pam.d/system-auth, /etc/pam.d/password-auth case - first option setting wins
    # (any following settings for the same option are ignored) => operation doesn't matter
    # if it's 'None' / 'equals' / 'greater than or equal' under assumption instance == '1'
    if filepath == '/etc/pam.d/system-auth' or filepath == '/etc/pam.d/password-auth' and instance == '1':
        return 'PASS'

    # /etc/sysconfig/iptables, /etc/sysconfig/ip6tables case - first option setting wins
    # (any following setting for the same option is ignored)
    # From http://fedoraproject.org/wiki/How_to_edit_iptables_rules#Listing_Rules
    # "Note that Rules are applied in order of appearance, and the inspection ends
    #  immediately when there is a match. Therefore, for example, if a Rule rejecting
    #  ssh connections is created, and afterward another Rule is specified allowing ssh,
    #  the Rule to reject is applied and the later Rule to accept the ssh connection is not."
    if filepath == '/etc/sysconfig/iptables' or filepath == '/etc/sysconfig/ip6tables' and instance == '1':
        return 'PASS'

    # Return failure by default
    return 'FAIL'

def main():

    if len(sys.argv) < 2:
        print "Provide path to OVAL xml file to be checked."
        sys.exit(1)

    PASS_COUNT = 0
    oval_file = sys.argv[1]

    tree = ET.parse(oval_file)
    # Get OVAL XML root element
    root = tree.getroot()
    # Get OVAL XML <objects> element
    objects_root = root.findall('{' + oval_ns + '}objects')
    # Get the list of all <ind:textfilecontent54_object> elements
    tfc54_objects = objects_root[0].findall('{' + oval_ns + '#independent}textfilecontent54_object')
    print "Found %d objects to review:\n" % len(tfc54_objects)

    for obj in tfc54_objects:

        # Get 'id' attribute for object
        obj_id_attr = obj.get('id')
        # Get <ind:filepath> and <ind:path> + <ind:filename> subelements for object
        # If <ind:filepath> elem is None, object has <ind:path> + <ind:filename> combination
        # and vice versa
        obj_fp_elem = obj.find('{' + oval_ns + '#independent}filepath')
        obj_path_elem = obj.find('{' + oval_ns + '#independent}path')
        obj_filename_elem = obj.find('{' + oval_ns + '#independent}filename')
        # Get <ind:instance> subelement for obj
        obj_ins_elem = obj.find('{' + oval_ns + '#independent}instance')
        # Get 'operation' attribute from the <ind:instance> subelement yet
        obj_op_attr = obj_ins_elem.get('operation')

        # Construct the filepath value for object (either from <ind:filepath> value
        # or by concatenating <ind:path> + <ind:filename> values)
        if obj_fp_elem is not None:
            obj_filepath = obj_fp_elem.text
        else:
            obj_filepath = obj_path_elem.text + '/' + obj_filename_elem.text

        # Verify record correctness based on 'id', 'filepath / path + filename',
        # 'operation', and 'instance' values
        result = verify_record_sanity(obj_id_attr, obj_filepath, obj_op_attr, obj_ins_elem.text)

        if result == 'PASS':
            PASS_COUNT += 1

        # Print out the review result for this object
        print "* Identifier\t= %-*s" % (40, obj_id_attr)
        if obj_fp_elem is not None:
            print "  Filepath\t= %-*s" % (40, obj_fp_elem.text)
        else:
            print "  Path\t\t= %-*s" % (40, obj_path_elem.text)
            print "  Filename\t= %-*s" % (40, obj_filename_elem.text)
        if obj_op_attr is not None:
            print "  Operation\t= %-*s" % (40, obj_op_attr)
        else:
            print "  Operation\t= None"
        print "  Instance\t= %-*s" % (40, obj_ins_elem.text)
        print "  Result\t= %-*s" % (40, result)
        print

    print "Ratio of passing objects to all objects: %d / %d" % (PASS_COUNT, len(tfc54_objects))

if __name__ == "__main__":
    main()



#!/usr/bin/python2

# OpenSCAP as of version 1.2.8 doesn't support OCIL check system yet. For
# example attempt to build RHEL/6 SSG benchmark produces the following error:
#
# scap-security-guide/RHEL/6 ] make content
# ..
# OpenSCAP Error: Unknown document type: 'ocil-ssg.xml' [oscap_source.c:172]
#
# As a result of this, when building datastream OCIL component is put into
# <ds:extended-components> element rather than into <ds:checks> element. See [1]
# for further details
#
# [1] https://github.com/OpenSCAP/openscap/issues/364
#
# But [2] http://csrc.nist.gov/publications/nistpubs/800-126-rev2/SP800-126r2.pdf
# (page "Table 6 - ds-checks [Page #22]" and section "3.2.4.2" suggests OCIL
# check system should be placed into <ds:checks> element when building a datastream
#
# The current oscap behaviour leads to:
# * [3] https://github.com/OpenSCAP/scap-security-guide/issues/1101 and
# * [4] https://github.com/OpenSCAP/scap-security-guide/issues/1100
# issues when validating the produced datastream using the NIST SCAP content
# test suite
#
# Therefore this script is performing the following:
# * moving the refence of found OCIL component from <ds:extended-components>
#   datastream element to <ds:checks> datastream element (and removing the
#   original <ds:extended-components> element)
# * also moving content of the found OCIL component from <ds:extended-component>
#   datastream element to <ds:component> datastream element (and removing the
#   original <ds:extended-component> element)
#
# This is a temporary workaround for bug [1], fixing the issues [3] and [4].
#
# Example run:
#   $ ./datastream_move_ocil_to_ds_checks.py ssg-rhel6-ds.xml new-ds.xml

import os
import sys
import lxml.html
import lxml.etree as ET

xlink_ns = "http://www.w3.org/1999/xlink"
datastream_ns = "http://scap.nist.gov/schema/scap/source/1.2"
ocil_ns = "http://scap.nist.gov/schema/ocil/2.0"

def parse_xml_file(xmlfile):
    with open(xmlfile, 'r') as xml_file:
        filestring = xml_file.read()
        tree = ET.fromstring(filestring)
    return tree


def move_ocil_ref_from_ds_extended_components_to_ds_checks(datastreamtree, ocilcomp):
    # This routine moves reference to present OCIL component from <ds:extended-components>
    # datastream element to <ds:checks> element (as required by SCAP v1.2 standard)

    # Locate <ds:checks> element
    dschecks = datastreamtree.find(".//{%s}checks" % datastream_ns)
    # Express <xlink:href> tag in namespace + tag form
    hreftag = '{' + xlink_ns + '}' + 'href'
    if dschecks is not None:
        # Sanity check if OCIL component has '<xlink:href>' attribute
        if hreftag in ocilcomp.attrib:
            # Save old <xlink:href> attribute value of the OCIL component
            oldocilhref = ocilcomp.attrib[hreftag]
            # Replace 'ecomp' with 'comp' in <xlink:href> attribute
            # Turns e.g. 'scap_org.open-scap_ecomp_ocil-ssg.xml' into 'scap_org.open-scap_comp_ocil-ssg.xml'
            ocilcomp.attrib[hreftag] = oldocilhref.replace('ecomp', 'comp')
            # Insert the reference to OCIL component past the last child in current <ds:checks> element
            dschecks.insert(len(dschecks)+1, ocilcomp)
            # Locate <ds:extended-components> element in datastream
            extendedcomps = datastreamtree.find(".//{%s}extended-components" % datastream_ns)
            # Remove the <ds:extended-components> element from the datastream if it's empty
            if extendedcomps is not None and len(extendedcomps) == 0:
                extendedcomps.getparent().remove(extendedcomps)
                # Return '<xlink:href>' value for later reuse
                return oldocilhref

    # Default return value
    return None


def move_ocil_content_from_ds_extended_component_to_ds_component(datastreamtree, ocilxlinkhref):
    # This routine moves content of OCIL element from <ds:extended-component>
    # datastream element to <ds:component> element (as required by SCAP v1.2 standard)

    # Drop the leading '#' character from <xlink:href> to get OCIL component ID
    ocilextcompid = ocilxlinkhref[1:]
    # Locate the <ds:extended-component> having @id set to OCIL component ID
    extendedcomp = datastreamtree.find(".//{%s}extended-component[@id=\"%s\"]" % (datastream_ns, ocilextcompid))
    # Replace 'ecomp' for 'comp' to be used in new <ds:component> ID
    ocildscompid = ocilextcompid.replace('ecomp', 'comp')
    # Default value for 'timestamp' attribute
    timestamp = None
    # Verify <ds:extended-component> contains 'timestamp' attribute
    if 'timestamp' in extendedcomp.attrib:
        # Save the 'timestamp' attribute value
        timestamp = extendedcomp.get('timestamp')
    else:
        print("Unable to obtain 'timestamp' attribute value from <ds:extended-component>. Exiting.")
        sys.exit(1)

    # Get children elements of <ds:extended-component> containing OCIL content
    extchildren = extendedcomp.getchildren()
    # There should be just one OCIL subcomponent in <ds:extended-component>
    if len(extchildren) == 1:
        # Decode possible HTML entities present in OCIL component
        extnohtmlents = ET.tostring(extendedcomp, method='html')
        # Create new element tree from decoded HTML
        extcomptree = ET.fromstring(extnohtmlents)
        # Locate the OCIL subcomponent within that element tree
        ocilcomp = extcomptree.find(".//{%s}ocil" % ocil_ns)

    # Now we have got everything:
    # * future OCIL <ds:component> ID        --> ocildscompid
    # * future OCIL <ds:component> timestamp --> timestamp
    # * future OCIL <ds:component> content   --> ocilcomp
    # to be ables to create new <ds:component> for OCIL content
    ocildscomp = ET.Element('{' + datastream_ns + '}component',
                            attrib = { 'id' : ocildscompid, 'timestamp' : timestamp },
                            nsmap = {'ds' : datastream_ns})
    # Insert the OCIL content into newly created <ds:component>
    ocildscomp.insert(len(ocildscomp)+1, ocilcomp)

    # Next insert that newly created OCIL <ds:component> into the datastream, thus
    # Get previous sibling of <ds:extended-component> (IOW last <ds:component> in benchmark)
    lastexistingdscomp = extendedcomp.getprevious()
    if lastexistingdscomp is not None:
        # Append newly created OCIL <ds:component> as following sibling directly after
        # last existing <ds:component> element
        lastexistingdscomp.addnext(ocildscomp)
        # Sanity check if the appending succeeded
        if ocildscomp.getprevious() != lastexistingdscomp:
            print("Error trying to append new OCIL <ds:component> directly after last <ds:component> element.")
            sys.exit(1)

        # Finally remove the original <ds:extended-component> OCIL element (since
        # OCIL content has now been placed into new <ds:component> element)
        extendedcomp.getparent().remove(extendedcomp)


def main():

    if len(sys.argv) < 3:
        print("Provide input and output SCAP Security Guide source datastream files.")
        print("This script moves <ocil:ocil> datastream component from " +
              "<ds:extended-components> to <ds:checks>.")
        sys.exit(1)

    # Input datastream file
    indatastreamfile = sys.argv[1]
    # Output datastream file
    outdatastreamfile = sys.argv[2]
    # Datastream element tree
    datastreamtree = parse_xml_file(indatastreamfile)

    # Locate <ds:extended-components> element in datastream
    extendedcomps = datastreamtree.find(".//{%s}extended-components" % datastream_ns)
    if extendedcomps is not None:
        # Locate OCIL component within <ds:extended-components>
        ocilcomps = extendedcomps.xpath(".//ds:component-ref[contains(@id,'-ocil')]",
                                        namespaces={"ds" : datastream_ns})
        # SSG produces datastream with exactly one OCIL component
        if len(ocilcomps) != 1:
            print("Expecting exactly one OCIL component in datastream.")
            sys.exit(1)
        else:
            comp = ocilcomps[0]
            # Move reference of found OCIL component from <ds:extended-components> to <ds:checks> element
            # Return old value of <xlink:href> atrribute value of the OCIL component
            oldocilhref = move_ocil_ref_from_ds_extended_components_to_ds_checks(datastreamtree, comp)
            if oldocilhref is not None:
                # Move OCIL component content from <ds:extended-component> to <ds:component>
                # Also update the ID replacing 'ecomp' with 'comp'
                move_ocil_content_from_ds_extended_component_to_ds_component(datastreamtree, oldocilhref)
            else:
                print("Error trying to move OCIL component in datastream. Exiting")
                sys.exit(1)

        # Write the updated benchmark into output datastream file
        ET.ElementTree(datastreamtree).write(outdatastreamfile)
        sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python2

import re
import sys
import os
import idtranslate
import lxml.etree as ET

# This script requires two arguments: an "unlinked" XCCDF file and an ID name
# scheme. This script is designed to convert and synchronize check IDs
# referenced from the XCCDF document for the supported checksystems, which are
# currently OVAL and OCIL.  The IDs are to be converted from strings to
# meaningless numbers.


oval_ns = "http://oval.mitre.org/XMLSchema/oval-definitions-5"
oval_cs = "http://oval.mitre.org/XMLSchema/oval-definitions-5"
ocil_ns = "http://scap.nist.gov/schema/ocil/2.0"
ocil_cs = "http://scap.nist.gov/schema/ocil/2"
xccdf_ns = "http://checklists.nist.gov/xccdf/1.1"
cce_uri = "http://cce.mitre.org"


def parse_xml_file(xmlfile):
    with open(xmlfile, 'r') as xml_file:
        filestring = xml_file.read()
        tree = ET.fromstring(filestring)
        # print filestring
    return tree


def get_checkfiles(checks, checksystem):
    # Iterate over all checks, grab the OVAL files referenced within
    checkfiles = set()
    for check in checks:
        if check.get("system") == checksystem:
            checkcontentref = check.find("./{%s}check-content-ref" % xccdf_ns)
            checkcontentref_hrefattr = checkcontentref.get("href")
            # Include the file in the particular check system only if it's NOT
            # a remotely located file (to allow OVAL checks to reference http://
            # and https:// formatted URLs)
            if not checkcontentref_hrefattr.startswith("http://") and \
               not checkcontentref_hrefattr.startswith("https://"):
                checkfiles.add(checkcontentref_hrefattr)
    return checkfiles


def create_xccdf_id_to_cce_id_mapping(xccdftree):
    #
    # Create dictionary having form of
    #
    # 'XCCDF ID' : 'CCE ID'
    #
    # for each XCCDF rule having <ident system='http://cce.mitre.org'>CCE-ID</ident>
    # element set in the XCCDF document
    xccdftocce_idmapping = {}

    xccdfrules = xccdftree.findall(".//{%s}Rule" % xccdf_ns)
    for rule in xccdfrules:
        xccdfid = rule.get("id")
        if xccdfid is not None:
            identcce = rule.find("./{%s}ident[@system='http://cce.mitre.org']" % xccdf_ns)
            if identcce is not None:
                cceid = identcce.text
                xccdftocce_idmapping[xccdfid] = cceid

    return xccdftocce_idmapping


def add_cce_id_refs_to_oval_checks(ovaltree, idmappingdict):
    #
    # For each XCCDF rule ID having <ident> CCE set and
    # having OVAL check implemented (remote OVAL isn't sufficient!)
    # add a new <reference> element into the OVAL definition having the
    # following form:
    #
    # <reference source="CCE" ref_id="CCE-ID" />
    #
    # where "CCE-ID" is the CCE identifier for that particular rule
    # retrieved from the XCCDF file
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1092
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1230
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1229
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1228

    ovalrules = ovaltree.findall(".//{%s}definition" % oval_ns)
    for rule in ovalrules:
        ovalid = rule.get("id")
        ovaldesc = rule.find(".//{%s}description" % oval_ns)
        # Assign <reference> CCE elements only to those OVAL checks whose
        # XCCDF rule ID has <ident> CCE set
        if ovalid is not None and \
           ovaldesc is not None and \
           ovalid in idmappingdict:
            xccdfcceid = idmappingdict[ovalid]
            # IF CCE ID IS IN VALID FORM (either 'CCE-XXXX-X' or 'CCE-XXXXX-X'
            # where each X is a digit, and the final X is a check-digit)
            if re.search(r'CCE-\d{4,5}-\d', xccdfcceid) is not None:
                # Then append the <reference source="CCE" ref_id="CCE-ID" /> element right
                # after <description> element of specific OVAL check
                ccerefelem = ET.Element('reference', ref_id="%s" % xccdfcceid, source="CCE")
                ovaldesc.addnext(ccerefelem)
                # Sanity check if appending succeeded
                if ccerefelem.getprevious() is not ovaldesc:
                    print ("\n\tError trying to add CCE ID to %s. Exiting" % ovalid)
                    sys.exit(1)


def ensure_by_xccdf_referenced_oval_def_is_defined_in_oval_file(xccdftree, ovaltree):
    # Ensure all OVAL checks referenced by XCCDF are implemented in OVAL file
    # Drop the reference from XCCDF to OVAL definition if:
    # * Particular OVAL definition isn't present in OVAL file,
    # * That OVAL definition doesn't constitute a remote OVAL
    #   (@href of <check-content-ref> doesn't start with 'http'
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1092
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1095
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1098


    xccdfrules = xccdftree.findall(".//{%s}Rule" % xccdf_ns)
    for rule in xccdfrules:
        xccdfid = rule.get("id")
        if xccdfid is not None:
            # Search OVAL ID in OVAL document
            ovalid = ovaltree.find(".//{%s}definition[@id=\"%s\"]" % (oval_ns, xccdfid))
            if ovalid is None:
                # Search same ID in XCCDF document
                check = rule.find(".//{%s}check[@system=\"%s\"]" % (xccdf_ns, oval_ns))
                # Skip XCCDF rules not referencing OVAL checks
                if check is not None:
                    checkcontentref = check.find(".//{%s}check-content-ref" % xccdf_ns)
                    if checkcontentref is not None:
                        try:
                            checkcontentref_hrefattr = checkcontentref.get('href')
                        except KeyError:
                            # @href attribute of <check-content-ref> is required by XCCDF standard
                            print("\nError: Invalid OVAL <check-content-ref> detected! Exiting..")
                            sys.exit(1)

                        # Skip remote OVAL (should cover both 'http://' and 'https://' cases)
                        if checkcontentref_hrefattr.startswith('http'):
                            continue

                        # For local OVAL drop the reference to OVAL definition from XCCDF document
                        # in the case:
                        # * OVAL definition is referenced from XCCDF file,
                        # * But not defined in OVAL file
                        print("Removing <check-content> OVAL element for %s." % xccdfid)
                        rule.remove(check)


def check_and_correct_xccdf_to_oval_data_export_matching_constraints(xccdftree, ovaltree):
        # Verify if <xccdf:Value> 'type' to corresponding OVAL variable 'datatype' export matching constraint:
        #
        # http://csrc.nist.gov/publications/nistpubs/800-126-rev2/SP800-126r2.pdf#page=30&zoom=auto,69,313
        #
        # is met. Also correct the 'type' attribute of those <xccdf:Value> elements where necessary in
        # order the produced content to meet this constraint.
        #
        # To correct the constraint we use simpler approach - prefer to fix 'type' attribute of <xccdf:Value>
        # rather than 'datatype' attribute of the corresponding OVAL variable since there might be additional
        # OVAL variables, derived from the affected OVAL variable, and in that case we would need to fix the
        # 'datatype' attribute in each of them.
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1089


        # Define the <xccdf:Value> 'type' to OVAL variable 'datatype' export matching constraints mapping
        # as specified in Table 16 of XCCDF v1.2 standard:
        # http://csrc.nist.gov/publications/nistpubs/800-126-rev2/SP800-126r2.pdf#page=30&zoom=auto,69,313
        #
        oval_to_xccdf_datatype_constraints = {
            'int' : 'number',
            'float' : 'number',
            'boolean' : 'boolean',
            'string' : 'string',
            'evr_string' : 'string',
            'version' : 'string',
            'ios_version' : 'string',
            'fileset_revision' : 'string',
            'binary' :  'string'
        }

        # Loop through all <external_variables> in the OVAL document
        ovalextvars = ovaltree.findall(".//{%s}external_variable" % oval_ns)
        if ovalextvars is not None:
            for ovalextvar in ovalextvars:
                # Verify the found external variable has both 'id' and 'datatype' set
                if 'id' not in ovalextvar.attrib or 'datatype' not in ovalextvar.attrib:
                    print("\nError: Invalid OVAL <external_variable> found. Exiting")
                    sys.exit(1)
                # Obtain the 'id' and 'datatype attribute values
                if 'id' in ovalextvar.attrib and 'datatype' in ovalextvar.attrib:
                    ovalvarid = ovalextvar.get('id')
                    ovalvartype = ovalextvar.get('datatype')

                # Locate the corresponding <xccdf:Value> with the same ID in the XCCDF
                xccdfvar = xccdftree.find(".//{%s}Value[@id=\"%s\"]" % (xccdf_ns, ovalvarid))
                if xccdfvar is not None:
                    # Verify the found value has 'type' attribute set
                    if 'type' not in xccdfvar.attrib:
                        print("\nError: Invalid XCCDF variable found. Exiting")
                        sys.exit(1)
                    else:
                        xccdfvartype = xccdfvar.get('type')
                        # This is the required XCCDF 'type' for <xccdf:Value> derived
                        # from OVAL variable 'datatype' and mapping above
                        reqxccdftype = oval_to_xccdf_datatype_constraints[ovalvartype]
                        # Compare the actual value of 'type' of <xccdf:Value> with the requirement
                        if xccdfvartype != reqxccdftype:
                            # If discrepancy is found, issue a warning
                            warning = ("\nWarning: XCCDF 'type' of \"%s\" value does not meet the XCCDF "
                                       "value 'type' to OVAL variable 'datatype'\nexport matching constraint! "
                                       "Got: \"%s\", Expected: \"%s\". Resetting it! Set 'type' of \"%s\""
                                       "\n<xccdf:value> to '%s' directly in the XCCDF content to dismiss this warning!" %
                                       (ovalvarid, xccdfvartype, reqxccdftype, ovalvarid, reqxccdftype)
                                       )
                            print warning
                            # And reset the 'type' attribute of such a <xccdf:Value> to the required type
                            xccdfvar.attrib['type'] = reqxccdftype


def verify_correct_form_of_referenced_cce_identifiers(xccdftree):
    # Correct CCE identifiers have the form of
    # * either CCE-XXXX-X,
    # * or CCE-XXXXX-X
    # where each X is a digit, and the final X is a check-digit
    # based on http://people.redhat.com/swells/nist-scap-validation/scap-val-requirements-1.2.html Requirement A17
    #
    # But in SSG benchmarks the CCEs till unassigned have the form of e.g. "RHEL7-CCE-TBD"
    # (or any other format possibly not matching the above two requirements)
    #
    # If this is the case for specific SSG product, drop such CCE identifiers from the XCCDF
    # since they are in invalid format!
    #
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1230
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1229
    # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1228

    xccdfrules = xccdftree.findall(".//{%s}Rule" % xccdf_ns)
    for rule in xccdfrules:
        identcce = rule.find(".//{%s}ident[@system=\"%s\"]" % (xccdf_ns, cce_uri))
        if identcce is not None:
            cceid = identcce.text
            # Found CCE identifier doesn't have one of the allowed forms listed above
            if re.search(r'CCE-\d{4,5}-\d', cceid) is None:
                # Drop such <xccdf:ident> CCE element from the XCCDF in that case
                identcce.getparent().remove(identcce)


def main():
    if len(sys.argv) < 3:
        print "Provide an XCCDF file and an ID name scheme."
        print ("This script finds check-content files (currently, OVAL " +
               "and OCIL) referenced from XCCDF and synchronizes all IDs.")
        sys.exit(1)

    xccdffile = sys.argv[1]
    idname = sys.argv[2]

    os.chdir("./output")
    # Step over xccdf file, and find referenced check files
    xccdftree = parse_xml_file(xccdffile)

    # Create XCCDF rule ID to assigned CCE ID mapping
    xccdf_to_cce_id_mapping = create_xccdf_id_to_cce_id_mapping(xccdftree)

    # Check that OVAL IDs and XCCDF Rule IDs match
    if 'unlinked-ocilref' not in xccdffile:
        allrules = xccdftree.findall(".//{%s}Rule" % xccdf_ns)
        for rule in allrules:
            xccdf_rule = rule.get("id")
            if xccdf_rule is not None:
                checks = rule.find("./{%s}check" % xccdf_ns)
                if checks is not None:
                    for check in checks:
                        check_name = check.get("name")
                        # Verify match of XCCDF vs OVAL / OCIL IDs for
                        # * the case of OVAL <check>
                        # * the case of OCIL <check>
                        if (not xccdf_rule == check_name and check_name is not None \
                            and not xccdf_rule + '_ocil' == check_name \
                            and not xccdf_rule == 'sample_rule'):
                            print("The OVAL / OCIL ID does not match the XCCDF Rule ID!\n")
                            if '_ocil' in check_name:
                                print("\n  OCIL ID:       \'%s\'" % check_name)
                            else:
                                print("\n  OVAL ID:       \'%s\'" % check_name)
                            print("\n  XCCDF Rule ID: \'%s\'"
                                  "\n\nBoth OVAL and XCCDF Rule IDs must match!" % xccdf_rule)
                            sys.exit(1)

    checks = xccdftree.findall(".//{%s}check" % xccdf_ns)
    ovalfiles = get_checkfiles(checks, oval_cs)
    ocilfiles = get_checkfiles(checks, ocil_cs)

    if len(ovalfiles) > 1 or len(ocilfiles) > 1:
        sys.exit("referencing more than one file per check system " +
                 "is not yet supported by this script.")
    ovalfile = ovalfiles.pop() if ovalfiles else None
    ocilfile = ocilfiles.pop() if ocilfiles else None

    translator = idtranslate.idtranslator(idname)

    # Rename all IDs in the oval file
    if ovalfile:
        ovaltree = parse_xml_file(ovalfile)

        # Add new <reference source="CCE" ref_id="CCE-ID" /> element to those OVAL
        # checks having CCE ID already assigned in XCCDF for particular rule.
        # But add the <reference> only in the case CCE is in valid form!
        # Exit with failure if the assignment wasn't successful
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1092
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1230
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1229
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1228
        add_cce_id_refs_to_oval_checks(ovaltree, xccdf_to_cce_id_mapping)

        # Verify all by XCCDF referenced (local) OVAL checks are defined in OVAL file
        # If not drop the <check-content> OVAL checksystem reference from XCCDF
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1092
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1095
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1098
        ensure_by_xccdf_referenced_oval_def_is_defined_in_oval_file(xccdftree, ovaltree)

        # Verify the XCCDF to OVAL datatype export matching constraints
        # Fixes: https://github.com/OpenSCAP/scap-security-guide/issues/1089
        check_and_correct_xccdf_to_oval_data_export_matching_constraints(xccdftree, ovaltree)

        # Verify if CCE identifiers present in the XCCDF follow the required form
        # (either CCE-XXXX-X, or CCE-XXXXX-X). Drop from XCCDF those who don't follow it
        verify_correct_form_of_referenced_cce_identifiers(xccdftree)

        ovaltree = translator.translate(ovaltree, store_defname=True)
        newovalfile = ovalfile.replace("unlinked", idname)
        ET.ElementTree(ovaltree).write(newovalfile)

    # Rename all IDs in the ocil file
    if ocilfile:
        ociltree = parse_xml_file(ocilfile)
        ociltree = translator.translate(ociltree)
        newocilfile = ocilfile.replace("unlinked", idname)
        ET.ElementTree(ociltree).write(newocilfile)

    # Rename all IDs and file refs in the xccdf file
    for check in checks:
        checkcontentref = check.find("./{%s}check-content-ref" % xccdf_ns)

        # Don't attempt to relabel ID on empty <check-content-ref> element
        if checkcontentref is None:
            continue
        # Obtain the value of the 'href' attribute of particular
        # <check-content-ref> element
        checkcontentref_hrefattr = checkcontentref.get("href")

        # Don't attempt to relabel ID on <check-content-ref> element having
        # its "href" attribute set either to "http://" or to "https://" values
        if checkcontentref_hrefattr.startswith("http://") or \
           checkcontentref_hrefattr.startswith("https://"):
            continue

        if check.get("system") == oval_cs:
            checkid = translator.generate_id("{" + oval_ns + "}definition",
                                             checkcontentref.get("name"))
            checkcontentref.set("name", checkid)
            checkcontentref.set("href", newovalfile)
            checkexport = check.find("./{%s}check-export" % xccdf_ns)
            if checkexport is not None:
                newexportname = translator.generate_id("{" + oval_ns + "}variable",
                                                       checkexport.get("export-name"))
                checkexport.set("export-name", newexportname)

        if check.get("system") == ocil_cs:
            checkid = translator.generate_id("{" + ocil_ns + "}questionnaire",
                                             checkcontentref.get("name"))
            checkcontentref.set("name", checkid)
            checkcontentref.set("href", newocilfile)
            checkexport = check.find("./{%s}check-export" % xccdf_ns)
            if checkexport is not None:
                newexportname = translator.generate_id("{" + oval_ns + "}variable",
                                                       checkexport.get("export-name"))
                checkexport.set("export-name", newexportname)

    newxccdffile = xccdffile.replace("unlinked", idname)
    # ET.dump(xccdftree)
    ET.ElementTree(xccdftree).write(newxccdffile)
    sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/python2

import sys
import os
import re
import lxml.etree as etree

# SSG Makefile to official product name mapping
CHROMIUM = 'Google Chromium Browser'
FEDORA = 'Fedora'
FIREFOX = 'Mozilla Firefox'
JRE = 'Java Runtime Environment'
RHEL = 'Red Hat Enterprise Linux'
WEBMIN = 'Webmin'
DEBIAN = 'Debian'
RHEVM = 'Red Hat Enterprise Virtualization Manager'
FUSE = 'JBoss Fuse'

def map_product(version):
    """Maps SSG Makefile internal product name to official product name"""

    product_name = None

    if re.findall('chromium', version):
        product_name = CHROMIUM
    if re.findall('fedora', version):
        product_name = FEDORA
    if re.findall('firefox', version):
        product_name = FIREFOX
    if re.findall('jre', version):
        product_name = JRE
    if re.findall('rhel', version):
        product_name = RHEL
    if re.findall('webmin', version):
        product_name = WEBMIN
    if re.findall('debian', version):
        product_name = DEBIAN
    if re.findall('rhevm', version):
        product_name = RHEVM
    if re.findall('fuse', version):
        product_name = FUSE
    return product_name

def fix_is_applicable_for_product(platform, product):
    """Based on the platform dict specifier of the remediation script to determine if this
    remediation script is applicable for this product. Return 'True' if so, 'False'
    otherwise"""
    product_name = ''
    product_version = None
    match = re.search(r'\d+$', product)
    if match is not None:
        product_version = product[-1:]
        product = product[:-1]

    # Define general platforms
    multi_platforms = ['multi_platform_all',
                       'multi_platform_' + product ]

    # First test if platform isn't for 'multi_platform_all' or
    # 'multi_platform_' + product
    for mp in multi_platforms:
        if mp in platform and product in ['rhel', 'fedora']:
            return True

    # Get official name for product 
    if product_version is not None:
        product_name = map_product(product) + ' ' + product_version
    else:
        product_name = map_product(product)

    # Test if this is for the concrete product version
    for pf in platform.split(','):
        if product_name == pf.strip():
            return True

    # Remediation script isn't neither a multi platform one, nor isn't applicable
    # for this product => return False to indicate that
    return False


def get_available_remediation_functions():
    """Parse the content of "$(SHARED)/xccdf/remediation_functions.xml" XML
    file to obtain the list of currently known SCAP Security Guide internal
    remediation functions"""

    remediation_functions = []
    # Determine the relative path to the "/shared" directory
    shared_dir = os.getenv("SHARED")
    # Default location of XML file with definitions of remediation functions
    xmlfilepath = ''
    # If location of /shared directory is known
    if shared_dir is not None:
        # Construct the final path of XML file with remediation functions
        xmlfilepath = shared_dir + '/xccdf/remediation_functions.xml'
    if xmlfilepath == '':
        print("Error determinining location of XML file with definitions of remediation functions")
        sys.exit(1)

    with open(xmlfilepath) as xmlfile:
        filestring = xmlfile.read()
        remediation_functions = re.findall('(?:^|\n)<Value id=\"function_(\S+)\"', filestring, re.DOTALL)

    return remediation_functions


def expand_xccdf_subs(fix, remediation_functions):
    """For those remediation scripts utilizing some of the internal SCAP
    Security Guide remediation functions expand the selected shell variables
    and remediation functions calls with <xccdf:sub> element

    This routine translates any instance of the 'populate' function call in
    the form of:

            populate variable_name

    into

            variable_name="<sub idref="variable_name"/>"

    Also transforms any instance of some other known remediation function (e.g.
    'replace_or_append' etc.) from the form of:

            function_name "arg1" "arg2" ... "argN"

    into:

            <sub idref="function_function_name"/>
            function_name "arg1" "arg2" ... "argN"
    """

    # This remediation script doesn't utilize internal remediation functions
    # Skip it without any further processing
    if 'remediation_functions' not in fix.text:
        return
    # This remediation script utilizes some of internal remediation functions
    # Expand shell variables and remediation functions calls with <xccdf:sub>
    # elements
    else:
        pattern = '\n+(\s*(?:' + '|'.join(remediation_functions) + ')[^\n]*)\n'
        patcomp = re.compile(pattern, re.DOTALL)
        fixparts = re.split(patcomp, fix.text)
        if fixparts[0] is not None:
            # Split the portion of fix.text from fix start to first call of
            # remediation function into two parts:
            # * head        to hold inclusion of the remediation functions
            # * tail        to hold part of the fix.text after inclusion,
            #               but before first call of remediation function
            try:
                rfpattern = '(.*remediation_functions)(.*)'
                rfpatcomp = re.compile(rfpattern, re.DOTALL)
                _, head, tail, _ = re.split(rfpatcomp, fixparts[0], maxsplit=2)
            except ValueError:
                print("Processing fix.text for: %s rule" % fix.get('rule'))
                print("Unable to extract part of the fix.text after " +
                      "inclusion of remediation functions. Aborting..")
                sys.exit(1)
            # If the 'tail' is not empty, make it new fix.text.
            # Otherwise use ''
            fix.text = tail if tail is not None else ''
            # Drop the first element of 'fixparts' since it has been processed
            fixparts.pop(0)
            # Perform sanity check on new 'fixparts' list content (to continue
            # successfully 'fixparts' has to contain even count of elements)
            if len(fixparts) % 2 != 0:
                print("Error performing XCCDF expansion on remediation " +
                      "script: %s" % fix.get('rule'))
                print("Invalid count of elements. Exiting")
                sys.exit(1)
            # Process remaining 'fixparts' elements in pairs
            # First pair element is remediation function to be XCCDF expanded
            # Second pair element (if not empty) is the portion of the original
            # fix text to be used in newly added sublement's tail
            for idx in range(0, len(fixparts), 2):
                # We previously removed enclosing newlines when creating
                # fixparts list. Add them back and reuse the above 'pattern'
                fixparts[idx] = "\n%s\n" % fixparts[idx]
                # Sanity check (verify the first field truly contains call of
                # some of the remediation functions)
                if re.match(pattern, fixparts[idx], re.DOTALL) is not None:
                    # This chunk contains call of 'populate' function
                    if 'populate' in fixparts[idx]:
                        # Extract variable name
                        varname = re.search('\npopulate (\S+)\n',
                                            fixparts[idx], re.DOTALL).group(1)
                        # Define fix text part to contribute to main fix text
                        fixtextcontribution = '\n%s="' % varname
                        # Append the contribution
                        fix.text += fixtextcontribution
                        # Define new XCCDF <sub> element for the variable
                        xccdfvarsub = etree.SubElement(fix, "sub",
                                                       idref=varname)
                        # If second pair element is not empty, append it as
                        # tail for the subelement (prefixed with closing '"')
                        if fixparts[idx + 1] is not None:
                            xccdfvarsub.tail = '"' + '\n' + fixparts[idx + 1]
                        # Otherwise append just enclosing '"'
                        else:
                           xccdfvarsub.tail = '"' + '\n'
                        # Append the new subelement to the fix element
                        fix.append(xccdfvarsub)
                    # This chunk contains call of other remediation function
                    else:
                        # Extract remediation function name
                        funcname = re.search('\n\s*(\S+)(| .*)\n',
                                             fixparts[idx],
                                             re.DOTALL).group(1)
                        # Define new XCCDF <sub> element for the function
                        xccdffuncsub = etree.SubElement(fix, "sub",
                                                        idref='function_%s' % \
                                                        funcname)
                        # Append original function call into tail of the
                        # subelement
                        xccdffuncsub.tail = fixparts[idx]
                        # If the second element of the pair is not empty,
                        # append it to the tail of the subelement too
                        if fixparts[idx + 1] is not None:
                            xccdffuncsub.tail += fixparts[idx + 1]
                        # Append the new subelement to the fix element
                        fix.append(xccdffuncsub)
                        # Ensure the newly added <xccdf:sub> element for the
                        # function will be always inserted at newline
                        # If xccdffuncsub is the first <xccdf:sub> element
                        # being added as child of <fix> and fix.text doesn't
                        # end up with newline character, append the newline
                        # to the fix.text
                        if fix.index(xccdffuncsub) == 0:
                            if re.search('.*\n$', fix.text) is None:
                                fix.text += '\n'
                        # If xccdffuncsub isn't the first child (first
                        # <xccdf:sub> being added), and tail of previous
                        # child doesn't end up with newline, append the newline
                        # to the tail of previous child
                        else:
                            previouselem = xccdffuncsub.getprevious()
                            if re.search('.*\n$', previouselem.tail) is None:
                                previouselem.tail += '\n'

    # Perform a sanity check if all known remediation function calls have been
    # properly XCCDF substituted. Exit with failure if some wasn't

    # First concat output form of modified fix text (including text appended
    # to all children of the fix)
    modfixtext = fix.text
    for child in fix.getchildren():
        if child is not None and child.text is not None:
            modfixtext += child.text
    for f in remediation_functions:
        # Then efine expected XCCDF sub element form for this function
        funcxccdfsub = "<sub idref=\"function_%s\"" % f
        # Finally perform the sanity check -- if function was properly XCCDF
        # substituted both the original function call and XCCDF <sub> element
        # for that function need to be present in the modified text of the fix
        # Otherwise something went wrong, thus exit with failure
        if f in modfixtext and funcxccdfsub not in modfixtext:
            print("Error performing XCCDF <sub> substitution for function")
            print("%s in %s fix.\nExiting..." % (f, fix.get("rule")))
            sys.exit(1)


def main():
    if len(sys.argv) < 2:
        print "Provide a directory name, which contains the fixes."
        sys.exit(1)

    product = sys.argv[1]
    fixdir = sys.argv[2]
    output = sys.argv[3]

    fixcontent = etree.Element("fix-content", system="urn:xccdf:fix:script:sh",
                               xmlns="http://checklists.nist.gov/xccdf/1.1")
    fixgroup = etree.SubElement(fixcontent, "fix-group", id="bash",
                                system="urn:xccdf:fix:script:sh",
                                xmlns="http://checklists.nist.gov/xccdf/1.1")

    remediation_functions = get_available_remediation_functions()

    platform = {}
    included_fixes_count = 0
    for filename in os.listdir(fixdir):
        if filename.endswith(".sh"):
            # Create and populate new fix element based on shell file
            fixname = os.path.splitext(filename)[0]

            with open(fixdir + "/" + filename, 'r') as fix_file:
                # Assignment automatically escapes shell characters for XML
                script_platform = fix_file.readline().strip('#').strip().split('=')
                if len(script_platform) > 1:
                    platform[script_platform[0].strip()] = script_platform[1].strip()
                if script_platform[0].strip() == 'platform':
                    if fix_is_applicable_for_product(platform['platform'], product):
                        fix = etree.SubElement(fixgroup, "fix", rule=fixname)
                        fix.text = fix_file.read()
                        included_fixes_count += 1

                        # Expand shell variables and remediation functions into
                        # corresponding XCCDF <sub> elements
                        expand_xccdf_subs(fix, remediation_functions)
                else:
                    print("\nNotification: Removed the '%s' remediation script from merging as " \
                          "the platform identifier in the script is missing!" % filename)

    sys.stderr.write("\nNotification: Merged %d remediation scripts into XML document.\n" % included_fixes_count)
    tree = etree.ElementTree(fixcontent)
    tree.write(output, pretty_print=True)

    sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/python2

import sys
import csv
import re

# Define input template filename to resulting output filename mapping
files = { 'template_sysctl_static' : 'sysctl_static_',
          'template_sysctl_runtime' : 'sysctl_runtime_',
          'template_sysctl_ipv6' : 'sysctl_' }

files_var = { 'template_sysctl_static_var' : 'sysctl_static_',
	  'template_sysctl_runtime_var' : 'sysctl_runtime_',
          'template_sysctl_ipv6' : 'sysctl_' }

def output_checkfile(serviceinfo):
    # get the items out of the list
    sysctl_var, sysctl_val = serviceinfo
    # convert variable name to a format suitable for 'id' tags
    sysctl_var_id = re.sub('[-\.]', '_', sysctl_var)

    # if the sysctl value is not present, use the variable template
    if not sysctl_val.strip():
        # open the template files and perform the conversions
        for sysctlfile in files_var.keys():
            with open(sysctlfile, 'r') as templatefile:
                filestring = templatefile.read()
                filestring = filestring.replace("SYSCTLID", sysctl_var_id)
                filestring = filestring.replace("SYSCTLVAR", sysctl_var)
                # write the check
                with open("./output/" + files_var[sysctlfile] + sysctl_var_id +
                          ".xml", 'w+') as outputfile:
                    outputfile.write(filestring)
                    outputfile.close()

    else:
        # open the template files and perform the conversions
        for sysctlfile in files.keys():
            with open(sysctlfile, 'r') as templatefile:
                filestring = templatefile.read()
                filestring = filestring.replace("SYSCTLID", sysctl_var_id)
                filestring = filestring.replace("SYSCTLVAR", sysctl_var)
                filestring = filestring.replace("SYSCTLVAL", sysctl_val)
                # write the check
                with open("./output/" + files[sysctlfile] + sysctl_var_id +
                          ".xml", 'w+') as outputfile:
                    outputfile.write(filestring)
                    outputfile.close()


def main():
    if len(sys.argv) < 2:
        print ("Provide a CSV file containing lines of the format: " +
               "sysctlvariable,sysctlvalue")
        sys.exit(1)
    with open(sys.argv[1], 'r') as csv_file:
        # put the CSV line's items into a list
        sysctl_lines = csv.reader(csv_file)
        for line in sysctl_lines:

            # Skip lines of input file starting with comment '#' character
            if line[0].startswith('#'):
                continue

            output_checkfile(line)

    sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/python2

import sys
import csv
import re
import os


# Define input template filename to resulting output filename mapping
files = { 'template_sebool' : 'sebool_' }

PENDING = '>SEBOOL_BOOL</linux:pending_status'
CURRENT = '>SEBOOL_BOOL</linux:current_status'
EXTERN_VAR = '''</linux:selinuxboolean_state>

  <external_variable comment="external variable for %s"
  datatype="boolean" id="var_%s" version="1" />
''' 


def bool_state(sebool_state):
    sebool = ""
    sebool_state = re.sub(' ', '', sebool_state)

    if sebool_state == "on" or sebool_state == "enable":
        sebool_state = "enabled"
    elif sebool_state == "off" or sebool_state == "disable":
        sebool_state = "disabled"
    elif sebool_state == "use_var" or sebool_state == "":
        pass
    else:
        print("Error: Invalid SELinux state value: %s" % sebool_state)
        sys.exit()

    if sebool_state == "enabled":
        sebool = "true"
    if sebool_state == "disabled":
        sebool = "false"

    return sebool_state, sebool


def output_checkfile(serviceinfo):
    # get the items out of the list
    sebool_name, sebool_state = serviceinfo
    # convert variable name to a format suitable for 'id' tags
    sebool_id = re.sub('[-\.]', '_', sebool_name)

    # open the template files and perform the conversions
    (sebool_state, sebool_bool) = bool_state(sebool_state)
    (dirname, script) = os.path.split(__file__)

    if not sebool_state:
        pass
    else:
        for seboolfile in files.keys():
            with open(dirname + '/' + seboolfile, 'r') as templatefile:
                filestring = templatefile.read()
                filestring = filestring.replace("SEBOOLID", sebool_id)
                if sebool_state != "use_var":
                    filestring = filestring.replace("SEBOOL_BOOL", sebool_bool)
                else:
                    filestring = filestring.replace(CURRENT, ' var_ref="var_%s" /' % sebool_id)
                    filestring = filestring.replace(PENDING, ' var_ref="var_%s" /' % sebool_id)
                    filestring = filestring.replace("</linux:selinuxboolean_state>", EXTERN_VAR % (sebool_id, sebool_id))
                # write the check
                with open("./output/" + files[seboolfile] + sebool_id +
                          ".xml", 'w+') as outputfile:
                    outputfile.write(filestring)
                    outputfile.close()


def main():
    if len(sys.argv) < 2:
        print ("Provide a CSV file containing lines of the format: " +
               "seboolvariable,seboolstate")
        sys.exit(1)
    with open(sys.argv[1], 'r') as csv_file:
        # put the CSV line's items into a list
        sebool_lines = csv.reader(csv_file)
        for line in sebool_lines:

            # Skip lines of input file starting with comment '#' character
            if line[0].startswith('#'):
                continue

            output_checkfile(line)

    sys.exit(0)

if __name__ == "__main__":
    main()
